<?php
/**
 * @file
 * Style plugin for Semantic Panels.
 */

$plugin = array(
  'title' => t('Semantic'),
  'description' => t('Semantic Style.'),

  // Region.
  'render region' => 'semantic_panels_render_region',
  'settings form' => 'semantic_panels_region_settings_form',

  // Pane.
  'render pane' => 'semantic_panels_render_pane',
  'pane settings form' => 'semantic_panels_pane_settings_form',
  'pane settings form submit' => 'semantic_panels_pane_settings_form_submit',
);

/**
 * Region settings form.
 */
function semantic_panels_region_settings_form($settings) {
  $elements = _semantic_panels_get_elements();

  $form = array();

  $form['separator']['active'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add separator'),
    '#description' => t('Enable/disable separator between panes.'),
    '#default_value' => $settings['separator']['active'],
  );
  $form['separator']['element_type'] = array(
    '#title' => t('HTML element'),
    '#type' => 'select',
    '#options' => $elements,
    '#default_value' => $settings['separator']['element_type'],
    '#dependency' => array(
      'edit-settings-separator-active' => array(1),
    ),
    '#tree' => TRUE,
  );
  $form['separator']['show_css'] = array(
    '#type' => 'checkbox',
    '#title' => t('Create a CSS class'),
    '#default_value' => !empty($settings['separator']['show_css']),
    '#dependency' => array(
      'edit-settings-separator-active' => array(TRUE),
    ),
    '#tree' => TRUE,
  );
  $form['separator']['element_class'] = array(
    '#type' => 'textfield',
    '#title' => t('CSS class'),
    '#default_value' => $settings['separator']['element_class'],
    '#dependency' => array(
      'edit-settings-separator-show-css' => array(TRUE),
      'edit-settings-separator-active' => array(TRUE),
    ),
    '#dependency_count' => 2,
    '#tree' => TRUE,
  );

  return $form;
}

/**
 * Render region.
 */
function theme_semantic_panels_render_region($vars) {
  $settings = $vars['settings'];

  $output = '';
  $separator = '';

  if (!empty($settings['separator']) && $settings['separator']['active'] === 1) {
    $attributes = array();

    if (!empty($settings['separator']['show_css']) && !empty($settings['separator']['element_class'])) {
      $attributes['class'] = $settings['separator']['element_class'];
    }

    $separator = "<{$settings['separator']['element_type']}" . drupal_attributes($attributes) . "></{$settings['separator']['element_type']}>";
  }

  $output = implode($separator, $vars['panes']);

  drupal_alter('semantic_panels_region', $output, $vars);

  return $output;
}

/**
 * Pane settings form.
 */
function semantic_panels_pane_settings_form($settings) {
  $elements = _semantic_panels_get_elements();

  $form = array();

  // Element title.
  $form['element_title'] = array(
    '#type' => 'fieldset',
    '#title' => t('Title'),
    '#weight' => 0,
    '#collapsible' => TRUE,
  );
  $form['element_title']['type'] = array(
    '#title' => t('HTML element'),
    '#type' => 'select',
    '#options' => $elements,
    '#default_value' => $settings['element_title']['type'],
    '#fieldset' => 'element_title',
  );
  $form['element_title']['class_enable'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add CSS class(es)'),
    '#default_value' => !empty($settings['element_title']['class']),
    '#fieldset' => 'element_title',
  );
  $form['element_title']['class'] = array(
    '#type' => 'textfield',
    '#title' => t('CSS class(es)'),
    '#default_value' => $settings['element_title']['class'],
    '#dependency' => array(
      'edit-settings-element-title-class-enable' => array(TRUE),
    ),
    '#fieldset' => 'element_title',
  );
  _semantic_panels_get_attributes_fieldset($form,  $settings, 'element_title');

  // Element content.
  $form['element_content'] = array(
    '#type' => 'fieldset',
    '#title' => t('Content'),
    '#weight' => 1,
    '#collapsible' => TRUE,
  );
  $form['element_content']['type'] = array(
    '#title' => t('HTML element'),
    '#type' => 'select',
    '#options' => $elements,
    '#default_value' => $settings['element_content']['type'],
    '#fieldset' => 'element_content',
  );
  $form['element_content']['class_enable'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add CSS class(es)'),
    '#default_value' => !empty($settings['element_content']['class']),
    '#fieldset' => 'element_content',
  );
  $form['element_content']['class'] = array(
    '#type' => 'textfield',
    '#title' => t('CSS class(es)'),
    '#default_value' => $settings['element_content']['class'],
    '#dependency' => array(
      'edit-settings-element-content-class-enable' => array(TRUE),
    ),
    '#fieldset' => 'element_content',
  );
  _semantic_panels_get_attributes_fieldset($form, $settings, 'element_content');

  // Element wrapper.
  $form['element_wrapper'] = array(
    '#type' => 'fieldset',
    '#title' => t('Wrapper'),
    '#weight' => 2,
    '#collapsible' => TRUE,
  );
  $form['element_wrapper']['type'] = array(
    '#title' => t('HTML element'),
    '#type' => 'select',
    '#options' => $elements,
    '#default_value' => $settings['element_wrapper']['type'],
    '#fieldset' => 'element_wrapper',
  );
  $form['element_wrapper']['class_enable'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add CSS class(es)'),
    '#default_value' => !empty($settings['element_wrapper']['class']),
    '#fieldset' => 'element_wrapper',
  );
  $form['element_wrapper']['class'] = array(
    '#type' => 'textfield',
    '#title' => t('CSS class(es)'),
    '#default_value' => $settings['element_wrapper']['class'],
    '#dependency' => array(
      'edit-settings-element-wrapper-class-enable' => array(TRUE),
    ),
    '#fieldset' => 'element_wrapper',
  );
  _semantic_panels_get_attributes_fieldset($form, $settings, 'element_wrapper');

  // Default classes.
  $form['add_default_classes'] = array(
    '#type' => 'checkbox',
    '#title' => t('Add default classes'),
    '#default_value' => isset($settings['add_default_classes']) ? $settings['add_default_classes'] : 1,
    '#description' => t('If checked default classes will be added on every element. They will always be added to the wrapper element (to not break contextual links).'),
    '#weight' => 3,
  );

  return $form;
}

/**
 * Get "Other Attribute(s)" fieldset
 */
function _semantic_panels_get_attributes_fieldset(&$form, $settings, $element_key){
  $element_key_class =  str_replace('_', '-', $element_key);
  $attributes = _semantic_panels_get_attributes();

  $form[$element_key]['attributes'] = array(
    '#type' => 'fieldset',
    '#title' => t('Other attribute(s)'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
    '#description' => t('You can override the available attributes below by setting the <em>semantic_panels_attributes</em> variable.')
  );

  foreach($attributes as $attribute => $attribute_data){

    $form[$element_key]['attributes'][$attribute] = array(
      '#type' => 'fieldset',
      '#title' => $attribute,
      '#collapsible' => TRUE,
      '#collapsed' => TRUE
    );

    $form[$element_key]['attributes'][$attribute]['enabled'] = array(
      '#type' => 'checkbox',
      '#title' => t('Add <em>@attribute</em> attribute', array('@attribute' => $attribute)),
      '#default_value' => !empty($settings[$element_key]['attributes'][$attribute]['enabled']),
    );

    $form[$element_key]['attributes'][$attribute]['value'] = array(
      '#type' => 'textfield',
      '#title' => t("<em>@attribute</em> value",array('@attribute' => $attribute)),
      '#default_value' => $settings[$element_key]['attributes'][$attribute]['value'],
      '#dependency' => array(
        'edit-settings-' . $element_key_class . '-attributes-' . $attribute . '-enabled' => array(TRUE),
      ),
      '#description' => t('Value is not required')
    );

  }
  return $form;
}

/**
 * Pane settings form submit handler.
 */
function semantic_panels_pane_settings_form_submit($form, &$settings, $form_state) {

  foreach($settings as &$setting){
    if(!empty($setting['attributes'])){
      $setting['attributes_array'] = array();
      foreach($setting['attributes'] as $machine_name => $attribute_settings){
        if(empty($setting['attributes'][$machine_name]['enabled'])){
          // Don't save disabled attributes
          unset($setting['attributes'][$machine_name]);
        }else{
          // Build up $settings['attributes_array'] ready for drupal_attributes with only enabled attributes
          $setting['attributes_array'][$machine_name] = $setting['attributes'][$machine_name]['value'];
        }
      }
    }
  }
}

/**
 * Preprocess variables for semantic-panels-pane.tpl.php
 */
function template_preprocess_semantic_panels_pane(&$vars) {
  // Make $vars contain all the stuff that the normal panels_pane has.
  template_preprocess_panels_pane($vars);

  $pane_semantic_settings = $vars['settings'];

  // Title wrapper.
  $vars['title_html'] = '';

  if ($vars['title']) {
    $element_data = array(
      'content' => $vars['title'],
      'default_type' => 'h2',
      'default_classes' => 'pane-title',
    );

    $vars['title_html'] = _semantic_panels_get_html('element_title', $element_data, $pane_semantic_settings);
  }

  // Content wrapper.
  $vars['content_html'] = '';

  if ($vars['content']) {
    $element_data = array(
      'content' => $vars['content'],
      'default_type' => 'div',
      'default_classes' => 'pane-content',
    );

    $vars['content_html'] = _semantic_panels_get_html('element_content', $element_data, $pane_semantic_settings);
  }

  // Full pane wrapper.
  $vars['wrapper_type'] = _semantic_panels_get_type($pane_semantic_settings['element_wrapper']['type']);

  if(!empty($vars['wrapper_type'])){

    if (!empty($pane_semantic_settings['element_wrapper']['class'])) {
      $vars['classes_array'][] = $pane_semantic_settings['element_wrapper']['class'];
    }

    if (!empty($pane_semantic_settings['element_wrapper']['attributes_array'])) {
      $vars['attributes_array'] = array_merge((array) $vars['attributes_array'], $pane_semantic_settings['element_wrapper']['attributes_array']);
    }

  }
}

/**
 * Render pane.
 */
function theme_semantic_panels_render_pane($vars) {
  return theme('semantic_panels_pane', $vars);
}
